#!/usr/bin/env python

import numpy as np
from matplotlib import animation
from matplotlib import pyplot as plt
from matplotlib.backends.qt_compat import QtWidgets
from matplotlib.widgets import Slider

from logslider import LogSlider, LogSlider0

sπ = np.pi

ω0 = 1.0
β = 0.1

# ω1 = 0.1
# ω2 = 1.9
wω = 1
nω = 5001

ωω = np.logspace(-wω, wω, nω)

A = 10**1.05


def amplitude(ω, β):
    return 1.0 / (ω0**2 - ω**2 + 2j * β * ω)


def power(ω, β, A=A):
    if β == 0:
        result = np.zeros_like(ω)
        result[(nω - 1) // 2] = 2 * A
        return result
    bw2 = (2. * β * ω)**2
    return 0.25 / β * bw2 / ((ω0**2 - ω**2)**2 + bw2)


def amplitude_half_band(β):
    if 2. * β <= ω0:
        det = 2. * β * (3. * (ω0**2 - β**2))**0.5
        mid = ω0**2 - 2. * β**2
        half = 0.25 / (β * (ω0**2 - β**2)**0.5)
        return (np.max([mid - det, 0.])**0.5, (mid + det)**0.5), (half, half)
    else:
        return (0., (2. * (β**4 - β**2 * ω0**2 + ω0**4)**0.5 - 2. * β**2 + ω0**2)**0.5), (0.5, 0.5)


def power_half_band(β):
    mid = (ω0**2 + β**2)**0.5
    half = 0.125 / β
    return (mid - β, mid + β), (half, half)


fig, (ax1, ax2, ax3) = plt.subplots(nrows=3, sharex=True, gridspec_kw={'hspace': 0.05})

manager = fig.canvas.manager
manager.window.findChild(QtWidgets.QToolBar).setVisible(False)
manager.set_window_title("Drgania tłumione")
manager.full_screen_toggle()

ax1.axhline(0, color='0.7', lw=0.7)
ax1.axvline(ω0, color='0.7', lw=0.7)
ax2.axvline(ω0, color='0.7', lw=0.7)
ax3.axvline(ω0, color='0.7', lw=0.7)

ax2f = ax2.twinx()

aa = amplitude(ωω, β)
line1r, = ax1.plot(ωω, aa.real, lw=1.5, label='$A$ (elastyczna)')
line1i, = ax1.plot(ωω, -aa.imag, lw=1.5, label='$B$ (absorpcyjna)')
line2, = ax2.plot(ωω, np.abs(aa), lw=3.0, color='C2', label='$|C|$')
line2f, = ax2f.plot(ωω, np.arctan2(-aa.imag, aa.real) / np.pi, lw=1.5, color='C3')
ax2.plot([np.nan], [np.nan], lw=1.5, color='C3', label='$\\varphi$')
line2h, = ax2.plot(*amplitude_half_band(β), lw=0.7, color='C2')

line3, = ax3.plot(ωω, power(ωω, β), lw=1.5, color='C4')
line3h, = ax3.plot(*power_half_band(β), lw=0.7, color='C4')

ax1.set_ylim(-A, A)
ax2.set_ylim(0., A)
ax3.set_ylim(0., A**0.5)

ax2f.set_ylim(0., 1.)

ax1.set_ylabel("Amplitudy $A$, $B$")
ax2.set_ylabel("Amplituda $|C|$")
ax2f.set_ylabel("Faza $\\varphi$ [$\\pi$]")

ax3.set_ylabel("Moc absorbowana $\\langle P \\rangle$")

ax3.set_xlabel('Częstość $\\omega/\\omega_0$')
ax3.set_xlim(ωω[0], ωω[-1])
ax3.set_xscale('log')
# ax3.xaxis.set_major_locator(plt.MultipleLocator(0.1))

ax1.legend()
ax2.legend(loc='upper left')

fig.subplots_adjust(bottom=0.15, top=0.98, left=0.08, right=0.92)
βax = fig.add_axes([0.15, 0.01, 0.78, 0.03])
sax = fig.add_axes([0.96, 0.75, 0.02, 0.23])

sst = {
    'track_color': '0.7',
    'color': '0.7',
    'initcolor': 'black',
}

beta_slider = LogSlider0(
    ax=βax, label='Tłumienie $\\beta/\\omega_0$', valmin=-2, valmax=1, valstep=0.01, valinit=np.log10(2**(-0.5)), **sst
)
scale_slider = LogSlider(ax=sax, label='', valmin=0, valmax=2, valinit=1.02, valstep=0.02, **sst, orientation='vertical')

beta_slider.set_val(np.log10(β))


def on_slider(val):
    β = beta_slider.val
    aa = amplitude(ωω, β)
    if β == 0.0:
        aa[(nω - 1) // 2] = np.nan
    line1r.set_ydata(aa.real)
    line1i.set_ydata(-aa.imag)
    line2.set_ydata(np.abs(aa))
    line2f.set_ydata(np.arctan2(-aa.imag, aa.real) / np.pi)
    line2h.set_data(*amplitude_half_band(β))
    A = scale_slider.val
    line3.set_ydata(power(ωω, β, A))
    line3h.set_data(*power_half_band(β))
    ax1.set_ylim(-A, A)
    ax2.set_ylim(0., A)
    ax3.set_ylim(0., 0.5 * A)
    fig.canvas.draw_idle()


beta_slider.on_changed(on_slider)
scale_slider.on_changed(on_slider)

plt.show()
